/*
 * phononbackend.h
 *
 *  Created on: 05.12.2010
 *      Author: darkstar
 */

#ifndef PHONONBACKEND_H_
#define PHONONBACKEND_H_

#include <QObject>
#include <Phonon>

#include "addonmanager.h"
#include "playbackbackend.h"
#include "media.h"

class PhononBackendAddOn : public AddOn, public AbstractPlaybackBackendFactory
{
    Q_OBJECT
public:
    static PhononBackendAddOn& singleton();
#	define qPhononBackendAddOn PhononBackendAddOn::singleton()

    PhononBackendAddOn()
        :	AddOn()
    {
        qDebug("Registering PhononBackendAddOn");
    }

    virtual ~PhononBackendAddOn() {}

    virtual void init()
    {
        globalPlaybackBackendRegistry.registerFactory(this);
    }

    virtual void hookMainWindow(Quasar *mainWindow)
    {
        mainWindow_ = mainWindow;
    }

    /* AbstractPlaybackBackendFactory */

    virtual QString internalClassName() const { return "Phonon"; }
    virtual QString friendlyDisplayName() const { return "Phonon"; }

    virtual AbstractPlaybackBackend *createInstance();

private:
    Quasar *mainWindow_;
};

class PhononBackend: public AbstractPlaybackBackend
{
    Q_OBJECT
public:
    PhononBackend(QObject *parent = 0);
    virtual ~PhononBackend();

    virtual AbstractPlaybackBackend::PlayResult play(Media *);

    virtual bool isRunning() const;
    virtual bool isPlaying() const;
    virtual bool isPaused() const;

public slots:
    virtual void pause();
    virtual void stop();
    virtual void seekRelativeSeconds(int);
    virtual void seekAbsolutePercentage(int);
    virtual void seekAbsoluteSeconds(int);

private slots:
    void mediaObjectTotalTimeChanged(qint64);
    void mediaObjectStateChanged(Phonon::State, Phonon::State);
    void mediaObjectFinished();
    void mediaObjectTick(qint64);

private:
    Phonon::MediaObject *mediaObject_;
    Phonon::AudioOutput *audioOutput_;

    void debugPrintCaps();
};

#endif /* PHONONBACKEND_H_ */
